// {{{RME classifier 'Logical View::Battleship'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Battleship.h"
#endif

#include <RTSystem/World.h>
#include <Battleship.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"sonar"
	  , &SonarSignals::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Battleship_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Battleship_Actor( _rts, _ref );
}

const RTActorClass Battleship =
{
	(const RTActorClass *)0
  , "Battleship"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Battleship_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
};

#define SUPER RTActor

Battleship_Actor::Battleship_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

Battleship_Actor::~Battleship_Actor( void )
{
}

int Battleship_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// sonar
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &sonar;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void Battleship_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	timer.informEvery(RTTimespec( 1, 0) ); 
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J432B3819009A:sendPing'
INLINE_METHODS void Battleship_Actor::transition2_sendPing( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	sonar.ping().send();
	printf("Ping...\n");
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J432B381C00F6:swallowEcho'
INLINE_METHODS void Battleship_Actor::transition3_swallowEcho( const void * rtdata, SonarSignals::Base * rtport )
{
	// {{{USR
	printf("Echo\n");
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Battleship_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Battleship_Actor::chain2_sendPing( void )
{
	// transition ':TOP:Active:J432B3819009A:sendPing'
	rtgChainBegin( 2, "sendPing" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_sendPing( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Battleship_Actor::chain3_swallowEcho( void )
{
	// transition ':TOP:Active:J432B381C00F6:swallowEcho'
	rtgChainBegin( 2, "swallowEcho" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_swallowEcho( msg->data, (SonarSignals::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Battleship_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain2_sendPing();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'sonar'
				switch( signalIndex )
				{
				case SonarSignals::Base::rti_echo:
					chain3_swallowEcho();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Battleship_Actor::getActorData( void ) const
{
	return &Battleship_Actor::rtg_class;
}

const RTActor_class Battleship_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , Battleship_Actor::rtg_parent_state
  , &Battleship
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , Battleship_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId Battleship_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor Battleship_Actor::rtg_ports[] =
{
	{
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( Battleship_Actor, Battleship_Actor::timer )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"sonar"
	  , (const char *)0
	  , &SonarSignals::Base::rt_class
	  , RTOffsetOf( Battleship_Actor, Battleship_Actor::sonar )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
