// {{{RME classifier 'Logical View::Ocean'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Ocean.h"
#endif

#include <RTSystem/World.h>
#include <Ocean.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"sonarSignals"
	  , &SonarSignals::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Ocean_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Ocean_Actor( _rts, _ref );
}

const RTActorClass Ocean =
{
	(const RTActorClass *)0
  , "Ocean"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Ocean_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "outRange"
  , "inRange"
};

#define SUPER RTActor

Ocean_Actor::Ocean_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, inRangeTime( 3, 400000000 )
	, outRangeTime( 5, 300000000 )
{
}

Ocean_Actor::~Ocean_Actor( void )
{
}

int Ocean_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// sonarSignals
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &sonarSignals;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void Ocean_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	timer.informIn (outRangeTime);

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:outRange:Junction2:goInRange'
INLINE_METHODS void Ocean_Actor::transition2_goInRange( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timer.informIn (inRangeTime);

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:inRange:Junction2:goOutRange'
INLINE_METHODS void Ocean_Actor::transition3_goOutRange( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	timer.informIn (outRangeTime);

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:inRange:Junction3:sendEcho'
INLINE_METHODS void Ocean_Actor::transition5_sendEcho( const void * rtdata, SonarSignals::Conjugate * rtport )
{
	// {{{USR
	sonarSignals.echo().send();

	// }}}USR
}
// }}}RME

INLINE_CHAINS void Ocean_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Ocean_Actor::chain4_swallowPing( void )
{
	// transition ':TOP:outRange:Junction6:swallowPing'
	rtgChainBegin( 2, "swallowPing" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Ocean_Actor::chain2_goInRange( void )
{
	// transition ':TOP:outRange:Junction2:goInRange'
	rtgChainBegin( 2, "goInRange" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_goInRange( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void Ocean_Actor::chain5_sendEcho( void )
{
	// transition ':TOP:inRange:Junction3:sendEcho'
	rtgChainBegin( 3, "sendEcho" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_sendEcho( msg->data, (SonarSignals::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void Ocean_Actor::chain3_goOutRange( void )
{
	// transition ':TOP:inRange:Junction2:goOutRange'
	rtgChainBegin( 3, "goOutRange" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_goOutRange( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Ocean_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:outRange'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'sonarSignals'
				switch( signalIndex )
				{
				case SonarSignals::Conjugate::rti_ping:
					chain4_swallowPing();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain2_goInRange();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:inRange'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'sonarSignals'
				switch( signalIndex )
				{
				case SonarSignals::Conjugate::rti_ping:
					chain5_sendEcho();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_goOutRange();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Ocean_Actor::getActorData( void ) const
{
	return &Ocean_Actor::rtg_class;
}

const RTActor_class Ocean_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 3
  , Ocean_Actor::rtg_parent_state
  , &Ocean
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , Ocean_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 2
  , Ocean_Actor::rtg_Ocean_fields
};

const RTStateId Ocean_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
};

const RTPortDescriptor Ocean_Actor::rtg_ports[] =
{
	{
		"sonarSignals"
	  , (const char *)0
	  , &SonarSignals::Conjugate::rt_class
	  , RTOffsetOf( Ocean_Actor, Ocean_Actor::sonarSignals )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( Ocean_Actor, Ocean_Actor::timer )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor Ocean_Actor::rtg_Ocean_fields[] =
{
	// {{{RME classAttribute 'inRangeTime'
	{
		"inRangeTime"
	  , RTOffsetOf( Ocean_Actor, inRangeTime )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTTimespec
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'outRangeTime'
  , {
		"outRangeTime"
	  , RTOffsetOf( Ocean_Actor, outRangeTime )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTTimespec
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
