// {{{RME classifier 'Logical View::Server'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Server.h"
#endif

#include <RTSystem/TheSystem.h>
#include <Server.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"link"
	  , &Link::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Server_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Server_Actor( _rts, _ref );
}

const RTActorClass Server =
{
	(const RTActorClass *)0
  , "Server"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Server_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "waitForCommand"
  , "waitForTimer"
};

#define SUPER RTActor

Server_Actor::Server_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

Server_Actor::~Server_Actor( void )
{
}

int Server_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// link
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &link;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:waitForCommand:Junction2:commandRxd'
INLINE_METHODS void Server_Actor::transition2_commandRxd( const void * rtdata, Link::Conjugate * rtport )
{
	// {{{USR
	timer.informIn (RTTimespec (1, 0));

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:waitForTimer:Junction2:timeout'
INLINE_METHODS void Server_Actor::transition3_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	link.response().send();

	// }}}USR
}
// }}}RME

INLINE_CHAINS void Server_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Server_Actor::chain2_commandRxd( void )
{
	// transition ':TOP:waitForCommand:Junction2:commandRxd'
	rtgChainBegin( 2, "commandRxd" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_commandRxd( msg->data, (Link::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void Server_Actor::chain3_timeout( void )
{
	// transition ':TOP:waitForTimer:Junction2:timeout'
	rtgChainBegin( 3, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Server_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:waitForCommand'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'link'
				switch( signalIndex )
				{
				case Link::Conjugate::rti_command:
					chain2_commandRxd();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:waitForTimer'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Server_Actor::getActorData( void ) const
{
	return &Server_Actor::rtg_class;
}

const RTActor_class Server_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 3
  , Server_Actor::rtg_parent_state
  , &Server
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , Server_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId Server_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
};

const RTPortDescriptor Server_Actor::rtg_ports[] =
{
	{
		"link"
	  , (const char *)0
	  , &Link::Conjugate::rt_class
	  , RTOffsetOf( Server_Actor, Server_Actor::link )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( Server_Actor, Server_Actor::timer )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
